package oncore;

import java.util.*;
import java.sql.*;
import javax.servlet.http.*;
import java.io.*;
import javax.servlet.*;



/**
 * Class models instructor's session. This class has session as the base
 * class.
 */
public class InstrSession extends Session
{
    // The courses the instructor takes are fixed atleast 
    // through out a session. So get the list of courses(no&name) for
    // the instructor at the beginning and keep it in the session

    private Vector allCourseNo;
    private Vector allCourseName;

    /**
     * Constructor for the class InstrSession. 
     * This constructor executes a query on database to precompute and store
     * all course numbers and course names in vectors.
     * @param req This is HttpServletRequest got by doPost.
     * @param con This is database connection established by init of servlet.
     * @exception ValidationException Thrown when can't authenticate
     * @exception SQLException Thrown when some database problem occurs
     */
    public InstrSession(HttpServletRequest req, Connection con)	throws
        ValidationException, SQLException { super(req, con);

        // query to extract all courses for the instructor
        String query = "select course.cname, course.cno from course,faculty"
            +" where course.fno = faculty.fno"
            +" and course.fno = " + "'" + idno + "'";
        
        // #ifdef DEBUG
//        System.err.println(query);
        // #endif

	Statement stmt = conn.createStatement();
	ResultSet rscrs = stmt.executeQuery(query);
        allCourseNo = new Vector();
        allCourseName = new Vector();
        while(rscrs.next()) { 
            allCourseNo.addElement(rscrs.getString("cno"));
            allCourseName.addElement(rscrs.getString("cname"));
        }

    }

    /** Method to print the first page for instructor. This method does SQL querying
     * and collects all the data and presents it in html form. This method is
     * called in processQuery and also when the instructor logs in for the
     * first time.  
     * @param out PrintWriter to write output html to. 
     * @exception SQLException Thrown when some database problems occur.
     */
    public void printFirst(PrintWriter out) throws SQLException 
    { 
        int phase = Configuration.getPhase(); 
        out.println("<form name=instr_form" + " action=\"" 
                + Configuration.getMainUrl() + "\" method=post> ");
        // create a statement for executing JDBC query
        Statement stmt = conn.createStatement(); 
        out.println("<H1>Following are the course(s) for you<br>and the students" 
                +" currently registered for it</H1><br>") ; 

        out.println("<b> CHECK THE BOXES WHERE YOU WANT TO KHATCH THE STUDENT"
                +"</b><br>");

        //while(rscrs.next()) { // for each course
        for (int i=0;i<allCourseNo.size();i++) {
            String cname=(String)allCourseName.elementAt(i);
            String cno=(String)allCourseNo.elementAt(i);
            String attr_name = cno; 
//            System.err.println("course: " + cno + " " + cname);

            out.println("<H1>" + cno + " " + cname + "</H1><br>") ;
            String query = "select student.sname, registration.srollno from"
                    +" registration,student where"
                    +" registration.srollno=student.srollno and"
                    +" registration.cno='"+cno+"'";
                    

//            System.err.println(query);

            ResultSet rsstd = stmt.executeQuery(query);
            while (rsstd.next()) { // for each student
                
                // attribute name is cno_srollno
                String srollno = rsstd.getString("srollno");

                                
                String s = " <input type =checkbox name="+attr_name
                     + " value=" + srollno + ">" +
                    srollno+" <a href="+Configuration.getStudUrl()
                    +"?srollno="+srollno+">"+ rsstd.getString("sname")
                    +"</a><br>";
                out.println(s);

            }
        }


        stmt.close();

        // phase 2 code: provide extra checkboxes to facad to give consent
        if (phase == 2)
        {
            stmt = conn.createStatement();
            String query = "select student.sname , student.srollno,"
                + " regconsent.cno"
                + " from course, student, regconsent"
                + " where course.fno = '" + idno + "' and "
                + " student.srollno = regconsent.srollno"
                + " and regconsent.cno = course.cno"
                + " and regconsent.instr_cons = 0";

// #ifdef DEBUG
//            System.out.println(query);
// #endif
            
            ResultSet rset2 = stmt.executeQuery(query);

            out.println("<hr>");
            out.println("<h3>Phase 2:Give course consent to:</h3><br>");
            while (rset2.next())
            {
                String attr_name = "cons_" + rset2.getString("cno");
                String attr_value = rset2.getString("srollno");

                out.println(" <input type=checkbox"
                        +" name = " + attr_name + " value = "+ attr_value 
                        +" >"+"<a href="+Configuration.getStudUrl()+"?srollno="
                        +attr_value+">"+rset2.getString("sname") + "</a> " 
                        +" "+rset2.getString("cno")+" "
                        +rset2.getString("srollno")+"<br>");
            }
            stmt.close();
        }
        out.println("<input type=submit value=Submit> " +
                "<input type=reset value=\"Undo all\"><br></form>");
	   out.println("<form action=\"http://everest.cse.iitb.ernet.in:5024/servlet/fchoice \" method=\"post\">"+
			   "<input type=submit name=submit value=grading page>"+
			   "<input type=\"hidden\" name=\"formno\" value=\"ffr\"></form>");
			 

        return ;
    }

    /** Method to process a query from an html form. This method is called
     * every time some update query is sent by instructor. This method does
     * updates and deletes on database and then calls printFirst to show the
     * page.
     * @param req The HttpServletRequest got by method doPost in servlet
     * @param out PrintWriter to output html page. this is only passed to
     * printFirst.
     * @exception SQLException
     */
    public void processQuery(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {
        int phase = Configuration.getPhase();
        // for all the courses the instructor teaches, see if any student
        // from that course has been cancelled and correspondingly update
        // the database. (This way we ensure only the registrations 
        // for the courses for that instructor are modified).
        
        //first start preparing the query string
        // the cno=null is a dummy condition, just there so that "or ..."
        // condition can be appended in a loop
        String query = "delete from registration where cno=null ";
	Statement stmt = conn.createStatement();
        ResultSet rs;
        
        int numcrs = allCourseNo.size();
        for (int i=0;i<numcrs;i++) {
            String cno=(String)allCourseNo.elementAt(i);

            String cnameqry = "select cname from course where cno ='"
                            + cno + "'";
            Statement cnamestmt = conn.createStatement();
            ResultSet cnamers = cnamestmt.executeQuery(cnameqry);
            cnamers.next();
            String cname = cnamers.getString("cname");


            
            String[] droppedStuds = req.getParameterValues(cno);
            String[] givenConsentStuds = req.getParameterValues("cons_"+cno);

            int numstd = (droppedStuds == null) ? 0 : droppedStuds.length;
            int numconsents = (givenConsentStuds == null) ? 
                0 : givenConsentStuds.length;
                 
            for (int j=0;j<numconsents;j++)
            {
                String updt = "update regconsent " + " set "
                    +" regconsent.instr_cons=1 where regconsent.cno = '"
                    + cno + "' and regconsent.srollno = '"
                    + givenConsentStuds[j]+"'";
// #ifdef DEBUG
//                System.err.print(" Given consent to student number ("+j+") : " + givenConsentStuds[j] );
//                System.err.println(" Now chekcing if the facad has already" +" given him consent");
// #endif
                Statement updtstmt=conn.createStatement();
                updtstmt.executeUpdate(updt);

// Can safely remove the student from consent table and add her to
// registration table and send her mail.
                String checkqry = "select cno from regconsent "
                    +" where srollno = '"+givenConsentStuds[j]+ "' and "
                    +" instr_cons=1 and fac_ad_cons=1 ";
                Statement checkstmt = conn.createStatement();
                ResultSet checkrset = checkstmt.executeQuery(checkqry);
// #ifdef DEBUG
//                System.err.println("Executing query : "+checkqry);
// #endif
                if (checkrset.next())
                {
                    // for email id and student's name 
                    Statement stmt1 = conn.createStatement();
                    String mailq = "select sname,semail_id from student"
                              + " where srollno='" + givenConsentStuds[j]
                              + "'";
                    ResultSet rs1 = stmt1.executeQuery(mailq);
                    rs1.next();
// #ifdef DEBUG
//                System.err.println("Executing query : "+mailq);
// #endif
                    String emailid = rs1.getString("semail_id");
// #ifdef DEBUG
//                    System.err.println("Sending mail to "+emailid);
                    rs1.next();
// #endif
                    String consentmsg ="Subject: Online Course Registration\n"
                        +"Dear "+ rs1.getString("sname")
                        +", \n      You have been "
                        +"allowed by your faculty advisor and course "
                        +" instructor\n to take following course(s) :\n";
                    stmt1.close();

                    do {
                        String delqry = "delete from regconsent "
                            +" where srollno = '"+givenConsentStuds[j]
                            +"' and cno = '"+ checkrset.getString("cno")
                            +"'";
                        String insertqry = "insert into registration "
                            +" values ('"+givenConsentStuds[j]+"','"
                            +checkrset.getString("cno")+"')";
// #ifdef DEBUG
//                        System.err.println("Insert Query : "+ insertqry);
// #endif
// #ifdef DEBUG
//                        System.err.println("Delete Query : "+ insertqry);
// #endif
                        Statement delstmt = conn.createStatement();
                        Statement insstmt = conn.createStatement();
                        delstmt.executeUpdate(delqry);
                        insstmt.executeUpdate(insertqry);
                        insstmt.close();
                        delstmt.close();
                        consentmsg = 
                            consentmsg.concat(checkrset.getString("cno") +"\n");
                    } while ( checkrset.next());
                // now send a mail to the student. Hopefully the mesg is ok!
                    Mailer m = new Mailer(emailid,consentmsg);
                    m.start();
// #ifdef DEBUG
//                    System.err.println(" Message is "+consentmsg);
// #endif
                }
        }

            for (int j=0;j<numstd;j++) {
                // append to the query string..
                query = query + " or (cno='" + cno
                              + "' and srollno ='" +  droppedStuds[j] + "') ";
                
                
                rs = stmt.executeQuery("select sname,semail_id from student"
                                      + " where srollno='" + droppedStuds[j]
                                      + "'");
                rs.next();
                String emailid = rs.getString("semail_id");
                

                Mailer m = new Mailer(emailid,"Subject: Oncore: dropped from course "
                            + cno + "\n\n" +" Dear "+rs.getString("sname")
                            + "\n     You have been *dropped* from the course "
                            + cname + " ( " + cno + " ) " + "by the instructor for the course"
                            + " just now!! Please visit the ONCORE page "
                            + " to see your latest status\n\nThank you,\n"
                            + "--\nOncore Team");
                m.start();
            }
        }
	stmt.executeUpdate(query);  // later may report the result
        stmt.close();

        printFirst(out);
    }
}
