package oncore;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.lang.reflect.*;

import javax.servlet.*;
import javax.servlet.http.*;

/** Main servlet in the program. This handles almost all ( except the one
 * which request student information ) the requests. Any servicing of query
 * needs the user to have been authenticated before. Since HTTP is stateless
 * the state of authentication is maintained by cookies ( cookies are handled
 * by the class HttpServlet by itself, so that we need not write any code to
 * do that ). All the data which may depend on the context is read from
 * configuration file.
 */
public class Main extends HttpServlet
{
    // always store the connection to database with yourself so that you can
    // pass them around to sessions, validators and so on ...
    private Connection conn;
    
    /** Method which does initialisation stuff in the Servlet. This gets
     * hajjaar things  as startup parameters ( please refer he props file for
     * details ) and create a new Configuration object to initialize all the
     * fields of Configuration.
     * @param cfg This is ServletConfig object passed to the method by servlet
     * engine. This contains all initArgs.
     * @exception ServletException Thrown when kernel ( servlet ? ) panic ( ie
     * fatal error ;-) occurs in initialisation.
     */ 
    public void init(ServletConfig cfg) throws ServletException
    {
        // read configuration
        try {
            Configuration ocrcfg = new Configuration(cfg);
        } catch ( ConfigurationException ce ) {

            // couldn't read configuration file properly so one or more
            // parameters required almost everywhere are set properl. This
            // means that no point in continuing until configuration file is
            // fixed so show some error message and throw some exception to
            // exit.
            System.err.println("Error in configuration file");
            throw new ServletException();
        }
        
        try {
            // establish database connection through JDBC
            DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
            //                               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
            //     Is this system specific ? If yes how do we put it in config
            //     file ?
            conn = DriverManager.getConnection(Configuration.getJdbcHost(),
                       Configuration.getDblogin(),Configuration.getDbpasswd());
        } catch ( SQLException e) {

            // couldn;t connect to data so no point in running the servlet any
            // further so report error and throw some exception which will
            // cause the servlet to exit. Can you think of any better way ?
            // probably System.exit() if it exists.

            System.err.println("Couldn't connect to database");
            throw new ServletException();
        }
        
        System.out.println("Entering phase "+Configuration.getPhase());
    }

    /** Method to show starting page ( html form ). 
      * @param out print html things to this
      */
    void initialPage(PrintWriter out)
    {
        // check that a session is not associated with this
        // a give a warning or something
        // also prob.ly before doing processQuery check it's not
        // this page that is submitted
        out.println("<HTML><HEAD><title> This Way In! </TITLE></HEAD><BODY"
            +" TEXT=blue link=red"
            +" BACKGROUND=http://www.cse.iitb.ernet.in/~bhati/Proj/OnCoRe/oncbg.jpg>" 
        + "<h1><center><image alt=oncore src=http://www.cse.iitb.ernet.in/~bhati/Proj/OnCoRe/oncore.jpg height=100>"
        +"<form align=center action=\"http://everest.cse.iitb.ernet.in:5024/"
        // the port no. and server name should be read from a .rc file 
            +"servlet/Main\" method=post>" 
            +"<input type=hidden name=FormId value=Login>"
            +" Identification No:  "
            +"<input type=text size=15 maxlength=15 name=id_number>"
            +"<br> Password :<input type=password size=10 maxlength=10 name=password>"
            +"<br> Student <input type=radio name=whoisit value=student>"
            +" Facad <input type=radio name=whoisit value=facad>"
            +" Instructor <input type=radio name=whoisit value=instr>"
            +"<br> <input type=submit value=go> <input type=reset value=khatch> "
                +"</form></center></h1> </body> </HTML>");
    return;
    }

    /** Method which will get called when a user tries to access the servlet
     * for the first time. This method gets invoked when the browser sends
     * http request of the form GET http://.../servlet/Main HTTP/1.1. This
     * method just sends back the login page.
     * @param req The request from user which requests initial page.
     * @param res The response to be submitted to user.
     * @exception ServletException thrown when some problem with servlet
     * engine.
     * @exception IOException thrown when socket I/O goes wrong.
     */
    public void doGet(HttpServletRequest req,HttpServletResponse res) 
    throws ServletException, IOException
    {
        // set the content type in header
        res.setContentType("text/html");

        // get writer object for the result
        PrintWriter out = res.getWriter();
             
        // write the login page
        initialPage(out);
        out.close();
    }

    /** Method which gets called every time use submits some query. Servlet
     * engine will automatically invoke this method whenever the browser
     * submits the HTTP request in the format: 
     * POST http://.../servlet/Main HTTP/1.1
     * Very crucial method since this method decides to whom control should be
     * transferred.
     * @param req The request submitted to the servlet engine.
     * @param res The response to be submitted to the user.
     * @exception ServletException Thrown when some fatal error occurs with
     * the servlet.
     * @exception IOException Thrown when error in socket I/O occurs.
     */
    public void doPost(HttpServletRequest req,HttpServletResponse res) 
    throws ServletException, IOException
    {
        PrintWriter out = res.getWriter();
        res.setContentType ("text/html");
        out.println("<HTML><TITLE>Online Course Registration</TITLE>"
                    + "<BODY TEXT=blue link=red bgcolor=white>");

        // Have one httpsession associated with each session so that managing
        // cookies and identification of sessions will be managed by servlet
        // engine itself.
        HttpSession hs ;

        // check if the current form is Login Form 
 /*      if( (req.getParameter("FormId")!=null) && 
                ( "Login".compareTo(req.getParameter("FormId")) == 0) ){
            // if the form is login form then try to get back the httpsession
            // from Servlet request.
            hs = req.getSession(false);
            if (hs!=null) {
                // if the form was login form and there was an httpsession
                // associated with it then invalidate the session since when
                // the user submitts login form she should get a new session
                // and should not be allowed to do any keeda with older
      //          // session.
                hs.invalidate();
            }
        }*/
            
        // Now create a session if it does not exists already, if it exists
        // then retrieve it from httpServlet request.
        hs = req.getSession(true); 

        // get back our own session object from httpsession
        Session ss = (Session)hs.getValue("OurSession");

        if ( ss == null ) { // ie, a new session
            // whoisit can be "student", "facad" or "instr" 
            String whoisit = (String)hs.getValue("kaun"); 
          System.out.println("who is it: " + whoisit.toString());
            if (whoisit == null ) {
                // seems to be a form from the wrong place!
                out.println("<h1>Illegal/Incomplete Form</h1><br>"
                        + " Please try again<br>"
                        + "</BODY></HTML>");
                out.close();
                return;
            }

            // Create a new Session and put it inside the HttpSession
            try {

                // find out whether it's facad / instr / student
                if (whoisit.compareTo( "facuadv" ) == 0) {
                   ss = new FacadSession(req,conn);
			    out.println("faculty advisor");
                }
                else if (whoisit.compareTo( "instruc" ) == 0 ) {
                   ss = new InstrSession(req,conn);
                }
                else  if (whoisit.compareTo( "student"  ) == 0) 
                {
                   ss = new StudentSession(req,conn);
                }
                // Illegal Form
                else { 
                    out.println("<h1>Illegal Form</h1><br>"
                            + " Please try again<br>"
                            + "</BODY></HTML>");
                    out.close();
                    return;
                }

                // after creating a session save its value with httpsession.
                hs.putValue("OurSession",ss);
                try {
                    // show initial page for that session
                    ss.printFirst(out);
                } catch (SQLException s) {
                    out.println("Internal Server Error! Please try again later");
                }


            } catch (ValidationException v) {
//                System.err.println("Authentication Failed " +
//                        "or Session not valid any more");
                out.println("Authentication Failed " +
                        "or Session not valid any more! Please login again.");
                hs.invalidate();
            } catch (SQLException s) {
                out.println("Internal Server Error!! Please try again later");
                // hs.invalidate();
            }

        }
        else { // if (ss != null) initially
            try {
                // then process th equery submitted by the user
                ss.processQuery(req,out);
            }
            catch (SQLException s) {
                // should print some debug info into System.err/Logfile
                out.println("<h1>Invalid Form!</h1>"
                        +"Please go back to the form you submitted,"
                        +"<bf>reset</bf> it, and fill it again carefully.");
                // hs.invalidate();
            }

        }   // endif (ss == null)

        out.println("<li><a href="+Configuration.getUrl()+">Login</a>"
            +" <a href="+Configuration.getHelpUrl()+">Help!</a>");
        //    +" <a href="+Configuration.getGradingUrl()+">grading page</a>");
          //  +" <a href="+Configuration.getcreUrl()+">Credits!</a></BODY></HTML>");
        out.close();
        return;
                
    }
        
            
    public String getServletInfo()
    {
        return "Online Course Registration Server";
    }
}
