package oncore;

import java.util.*;
import java.sql.*;
import javax.servlet.http.*;
import java.io.*;
import javax.servlet.*;

/** Class to model a generic session for student, instructor and faculty
 * advisor. This class abstarcts out the authentication procedure which is
 * common to all of the sessions. 
 */
abstract public class Session 
{
    // store database connection so that all the subclasses can use it. Note
    // that having a common database connection across all the sessions won't
    // create concurrency issues since every session will be in different
    // thread and JAVA RunTime Environment will manage all the concurrent
    // accesses to the same object.
    Connection conn;

    // identification ( usually roll number ) for the person
    String idno;

    /** Constructor initializes variables and also authenticates the user of
     * the session. For authentication it uses Validator class. 
     * @param req HttpServletRequest object got by method doPost. This will be
     * used by the constructor to find out user name and passowrd in the form.
     * @param con DataBase connection
     * @exception ValidationException Thrown if user cannot be authenticated.

     */
    public Session(HttpServletRequest req, Connection con) 
        throws ValidationException
    {
        conn = con;

        // Validate the passwd etc.
	Validator v = new Validator(conn);

        // get id and password from the form.
	HttpSession hs=req.getSession(true);
	idno = hs.getValue("login_id").toString();
	String passwd = hs.getValue("passwd").toString();
	String who = hs.getValue("kaun").toString();
	System.out.println("validating session for "+idno+" passwd="+passwd);

//        System.err.println("Session mesg: password entered: " + passwd);
        try {
            // authenticate the user
            if(v.isValid(idno,passwd,who,this instanceof StudentSession) != true)
            {
//                System.err.println("Throwing ValidationException");
                throw new ValidationException();
            }
        } catch (SQLException sqe) {
            // DEBUG
//            System.err.println("Caught SQLException.");
            throw new ValidationException();
        }
    }
    
    /** Method which will be called to print the html page for that particular
     * session. This method is implemented in the subclasses of this class.
     * This method will be directly called first time user logs in and
     * afterwards from the method processQuery to print the html.
     * @param out This is used to print the html contents of the page, which
     * are passed back to the http server / servlet engine.
     * @exception SQLException thrown if there is some problem with database
     * query,...
     */
    abstract public void printFirst(PrintWriter out) throws SQLException;

    /** Method which processes query submitted by the user. After processing
     * the query it may use ( in fact it uses ) the method printFirst to
     * actually display the results back.
     * @param req HttpServletRequest object received by doPost. This is used
     * to extract the form fields. This way the method doPost doesn't have to
     * know what fields to expect and where to pass them.
     * @exception SQLException thrown if there is some problem with database
     * query,...
     */
    abstract public void processQuery(HttpServletRequest req,PrintWriter out) 
        throws SQLException ;
}
