package oncore;
import java.lang.*;
import java.util.*;
import java.sql.*;
import java.io.*;
import javax.servlet.http.*;

/** Class to model a student session. This class has session as base class
  */
public class StudentSession extends Session
{
    // we precompute and store the group number so that it doesn't need to be
    // calculated every time.
    private String gpno;
    
    /** Makes a new StudentSession fot the request req. Sets the member semno  
      * by reading the Configuration. This constructor executes a query on
      * database to precompute and store the group number of the student.
      * @param req This is HttpServletRequest got by doPost.
      * @param con Database connection established by the servlet.
      * @exception ValidationException Thrown when authentication fails.
      * @exception SQLException Thrown because of some database problems while
      * trying to authenticate the student.
     */
    public StudentSession(HttpServletRequest req, Connection con)	
        throws ValidationException,SQLException
    {
        super(req, con);
        // calculate the gpno 
        Calendar gc = GregorianCalendar.getInstance();
        int year = gc.get(Calendar.YEAR);
        int month = gc.get(Calendar.MONTH);
        int sem ,semno;

        // Following piece of code finds out which is sem currently for
        // which the registration is being done. 
        sem = Configuration.getMonth_sem()[month]; // can be 0, 1 or 2
        
        String qyear = "select batch.yyyyofentry,batch.programme,batch.dno" 
            +" from batch,student where student.batchno = batch.batchno and"
            +" student.srollno = '" + idno + "'";
        // #ifdef DEBUG
        //System.err.println("Executing query : "+qyear);
        // #endif

	Statement stmt = conn.createStatement();
        ResultSet rs = stmt.executeQuery(qyear);
        rs.next();
        semno =  sem + 2*(year - rs.getInt("yyyyofentry"));

        String gpnoQuery = "select gp_no from "
            + "gp where "
            +" gp.dno = '" + rs.getString("dno")
            + "' and gp.programme = '" +
            rs.getString("programme") + "' and gp.sem = "+semno;

        
        //System.err.println("Executing GP query : "+gpnoQuery);
        rs = stmt.executeQuery(gpnoQuery);

        if (rs.next()) {
            gpno = rs.getString("gp_no");
        }
        else { // no group for this student!
            gpno = null; // this means no group
        }
        stmt.close();

    }


    /** Method to print a course name with a check box. Expects a resultset
     * with fields cno and cname and curl and furl and name and a boolean
     * pressed (to indicate check box status).
     * @param out PrintWriter object to write html page to.
     * @param rs Result set of the query to get course written in html form. 
     * @exception SQLException Thrown if the resultset is buggy.
     */
    private void printCourse(PrintWriter out, ResultSet rs, boolean pressed)
        throws SQLException
    {
         out.println(" <INPUT"
                +" type=checkbox name= registration value="
                +rs.getString("cno") +
                (pressed? " checked" : "") + " >"
                + "<a href=" + rs.getString("curl") + ">"
                + rs.getString("cno")+" "
                + rs.getString("cname") + "</a> "
                + "<a href=" + rs.getString("furl") + ">"
                + rs.getString("name") + "</a><br>");
	    		
    }


/** a function to print a course name WITHOUT a check box, used in phase 2. 
  * expects a resultset with fields cno and cname and curl and furl and name.
  * @param out PrintWriter object to write html stuff to.
  * @param rs resultset of the query from which courses an dinfo about them
  * can be extracted.
  * @exception SQLException Thrown if the resultset is buggy.
  */
    private void printCourse2(PrintWriter out, ResultSet rs )
        throws SQLException
    {
         out.println( "<a href=" + rs.getString("curl") + ">"
                + rs.getString("cno")+" "
                + rs.getString("cname") + "</a> "
                + "<a href=" + rs.getString("furl") + ">"
                + rs.getString("name") + "</a><br>");
    }

    /** Method to print default courses for the group to which the student
     * belongs in phase 0. Will be called if there are no entries for 
     * the student in registration table.        
     * @param out PrintWriter object to write html to.
     * @exception SQLException Thrown if database access creates some problem.
     */
    private void printDefault(PrintWriter out)
        throws SQLException
    {
        if (gpno==null)  {
            out.println("No Default course list available<br>");
            return;
        }
        
        // find the default courses using gpno
        String coreCourseQry = "select course.cno,course.cname,course.curl,"
            + " faculty.furl,faculty.name from "
            +" course,gp_defaults,faculty where course.cno = gp_defaults.cno and "
            +" gp_defaults.gp_no = '" + gpno + "'"
            +" and faculty.fno = course.fno " ;

        // #ifdef DEBUG 
//        System.err.println("Executing query : "+coreCourseQry);
        // #ifdef DEBUG

	Statement stmt = conn.createStatement();
        ResultSet coreSet = stmt.executeQuery(coreCourseQry);

        //generate the form here
        while(coreSet.next())
        {
            printCourse(out,coreSet,false);
        }

    }
    
/*-------    Below are some functions used in phase 0 to print the HTML page.
 *           The functions are print{Most,Search,End}. 
 *           printFirst is for all phases and is the one called when 
 *           the session starts off
 */
    
    /** Prints the headers, the courses currently registered for
        ( or the default list if not registered for any ) in phases 0,1.
        Doesn't close the form, and others can append to it
        Note: This is For the Main Page of the student session. phase 0 and 1
        method.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
     */
    private void printMost(PrintWriter out) throws SQLException
    {
	// create a statement for executing JDBC query
	Statement stmt = conn.createStatement();
	
        out.println("<form name=student_form"
           + " action=\"http://everest.cse.iitb.ernet.in:5024/servlet/Main\" "
           + " method=post> <br> " 
           + " <input type=hidden name=FormId value=StudentMain>"
           + " <h1>Your courses</h1><br>");
        out.println("<h2> Course Number and Course name </h2><br>");
	// Following is query string to get default course list
        String query = "select course.cno,course.cname,course.curl, faculty.furl,faculty.name"
            +" from course,registration,faculty"
            +" where course.cno = registration.cno and registration.srollno = "
            + "'" + idno + "'"
            +" and faculty.fno = course.fno ";
        // #ifdef DEBUG
        //System.err.println(query);
        // #endif

	ResultSet rset = stmt.executeQuery(query);

        //check if the guy has registered for some courses previously 
        if(rset.next() != false) 
        {
            do
            {
                printCourse(out,rset,true);
            } while(rset.next());
        }
        
        // if guy comes for first time then search default courses for him as
        // given in coursebook
        else
        {
            printDefault(out); 
        }
        stmt.close();
	return ;
    }


   /** For phase 2: Prints the courses currently registered for  (W/O CHECKBOXES)
      Also gives a search box. This does most of the stuff for phase 2.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
   */
    private void printMost2(PrintWriter out) throws SQLException
    {
	
        out.println(" <h1>The courses you have registered for:</h1><br>");
        out.println("<h2> Course Number and Course name </h2><br>");
	// Following is query string to get default course list
        String query = "select course.cno,course.cname,course.curl, faculty.furl,faculty.name"
            +" from course,registration,faculty"
            +" where course.cno = registration.cno and registration.srollno = "
            + "'" + idno + "'"
            +" and faculty.fno = course.fno ";
        // #ifdef DEBUG
        //System.err.println(query);
        // #endif

	// create a statement and rset for executing JDBC query
	Statement stmt = conn.createStatement();
	ResultSet rset = stmt.executeQuery(query);

        //check if the guy has registered for some courses previously 
        if(rset.next() != false) 
        {
            do
            {
                printCourse2(out,rset);
            } while(rset.next());
        }
        

        // if not registered for anything!!
        else
        {
            out.println("You have NOT registered for any course, and it's "
                    + "the last phase of registration already! --- Buck up man, wanna drop a sem or "
                    + "what? ;-)");
        }


        // the courses for which consent request was sent: and the consent status

        // only facad consent pending
        query = "select course.cno, course.cname, course.curl, faculty.name, faculty.furl from"
                + " course, regconsent, faculty where"
                + " regconsent.srollno = '" + idno + "'"
                + " and regconsent.cno = course.cno"
                + " and course.fno = faculty.fno"
                + " and instr_cons <> 0 "
                + " and fac_ad_cons = 0 ";
	rset = stmt.executeQuery(query);
        
        // check if there is a course for which consent is pending from facad
        if (rset.next() ) {

            out.println("<hr><h2>For Following course(s) only Facad's "
                 + " Consent is pending:</h2><br>");
            do
            {
                printCourse2(out,rset);
            } while(rset.next());
        }
        
        // only instr consent pending
        query = "select course.cno, course.cname, course.curl, faculty.name, faculty.furl from"
                + " course, regconsent, faculty where"
                + " regconsent.srollno = '" + idno + "'"
                + " and regconsent.cno = course.cno"
                + " and course.fno = faculty.fno"
                + " and instr_cons = 0 "
                + " and fac_ad_cons <> 0 ";
	rset = stmt.executeQuery(query);
        if (rset.next() ) {

            out.println("<hr><h2>For Following course(s) only Instructor's "
                 + " Consent is pending:</h2><br>");

            do
            {
                printCourse2(out,rset);
            } while(rset.next());
        }
        
        
        // both facad and instr consent pending
        query = "select course.cno, course.cname, course.curl, faculty.name, faculty.furl from"
                + " course, regconsent, faculty where"
                + " regconsent.srollno = '" + idno + "'"
                + " and regconsent.cno = course.cno"
                + " and course.fno = faculty.fno"
                + " and instr_cons = 0 "
                + " and fac_ad_cons = 0 ";
	rset = stmt.executeQuery(query);
        if (rset.next() ) {

            out.println("<hr><h2>For Following course(s) Instructor's and"
                 + " Facad's Consents are pending:</h2><br>");
            do
            {
                printCourse2(out,rset);
            } while(rset.next());
        }


        // no more queries 
        stmt.close();


        //the search box part
        // Open the search form:
        out.println("<hr><form name=student_form"
           + " action=\"http://everest.cse.iitb.ernet.in:5024/servlet/Main\" "
           + " method=post> <br> " 
           + " <input type=hidden name=FormId value=Search>");

        out.println ("If you want to register for any other courses"
                    + " you are required to get the consent of your "
                    + " Faculty Advisor and the Instructor. Use the"
                    + " search boxes below for it.<br>");
        out.println("Search a course number: Type in the course number here" 
                + " <input type=text name=cno_search"
                + " size=7 maxlength=10> <br>");

        out.println("Search a course name: Type in the course name here" 
                + " <input type=text name=cname_search"
                + " size=10 maxlength=40> <br>"); 
        // now close the SEARCH form, no course reg form here.

        out.println("<input type=submit value=Search> " +
                "<input type=reset type=reset value=\"Reset Form\"><br></form>");

        // Also show the Warnings about conflicts
        out.println("<hr>");
        printWarnings(out);
	return ;
    }


    
    
    /**  to print the end of the form, with the search box, for phase 0 
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
        */
    private void printEnd(PrintWriter out) throws SQLException
    {
        //the search part
        out.println("Search a course number: Type in the course number here" 
                + " <input type=text name=cno_search"
                + " size=7 maxlength=10> <br>");

         out.println("Search a course name: Type in the course name here" 
                + " <input type=text name=cname_search"
                + " size=10 maxlength=40> <br>"); 
        // now close the form.

        out.println("<input type=submit value=Submit> " +
                "<input type=reset type=reset value=\"Reset Form\"><br></form>");

	   out.println("<form action=\"http://everest.cse.iitb.ernet.in:5024/servlet/schoice \" method=\"post\">"+
			   "<input type=submit name=submit value=grading page>"+
			  "<input type=\"hidden\" name=\"formno\" value=\"sfr\"></form>");
	return ;
    }
    
    /** To print the Elective Names as Separate simple forms in phase 0.
        The (hidden) field FormId is "ElectiveMain" and the form name
        is the elective_name.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
     */
    private void printElectives(PrintWriter out) throws SQLException
    {
        // Following is query string to get the elective names for the group
        String query = "select elective_name.ename, elective_name.eno from gp_electives, elective_name"
            +" where elective_name.eno = gp_electives.eno and"
            +" gp_no = " + "'" + gpno + "'";

        Statement  stmt = conn.createStatement();
	ResultSet rset = stmt.executeQuery(query);

        out.println("<hr><h2>Electives<br></h2>");
        while(rset.next()) {

            /* The Form: The hidden field elecno is used to identify the Elective
             * and the submit field, named elecname is set to the name of the elective.
             * the field FormId is set to "ElectiveMain" (ie, in Student's main 
             * page, forms for electives
             */
            String elecname =  rset.getString("ename");
            String elecno =  rset.getString("eno");
            out.println("<form name=" + elecname
              + " action=\"http://everest.cse.iitb.ernet.in:5024/servlet/Main\" "
              + " method=post> <br> " 
              + " <input type=hidden name=FormId value=ElectiveMain>"
              + " <input type=hidden name=elecno value=" + elecno + ">"
              + " <input type=submit name=elecname value=" + elecname + ">"
              + " <br> </form>");

            // if (phase == 0)
            // print the current preferences
            // this query returns course info, ename, prefno.
            String query2 = "select course.cno, course.cname, course.curl, faculty.furl,faculty.name,"
                + " elective_prefs.prefno,elective_name.ename" 
                + " from elective_prefs, elective_name, course, faculty "
                + " where elective_prefs.eno = elective_name.eno and"
                + " elective_prefs.srollno = '" + idno + "' and "
                + " elective_prefs.eno = '" + elecno + "'"
                + " and elective_prefs.cno = course.cno "
                + " and faculty.fno = course.fno "
                + " order by prefno asc ";

            Statement  stmt2 = conn.createStatement();
            //System.err.println(query2);
            ResultSet rset2 = stmt.executeQuery(query2);
            while (rset2.next()) {
                out.println(rset2.getString("prefno")+". "
                + " <a href=" + rset2.getString("curl") + ">"
                + rset2.getString("cno") + " " + rset2.getString("cname") + "</a>"
                + " <a href=" + rset2.getString("furl") + ">"
                + rset2.getString("name") + "</a><br>");
            }
            stmt2.close();
        }         
        stmt.close();
        return;              
    }
    

    /** to print the search results. called by do_and_printSearch 
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void printSearch(PrintWriter out,ResultSet searset) throws SQLException
    {

        // print the list of courses
        out.println("<h3> Your search results are as follows:</h3><br>");
        
        // the search results
        if (searset.next()) {
            do {
                printCourse(out,searset,false);
            } while(searset.next());
        }
        else {
            out.println("No matches found<br>");
        }
        
	return ;

    }

    /** To send a page for the courses in an elective in phase 0.
      * The request comes through a form in the "Student Main Page".
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
     */
    private void sendElective(HttpServletRequest req,PrintWriter out) 
        throws SQLException
    {
        /* assuming a valid request */

        /* The Form Recvd: The hidden field elecno is used to identify the Elective
         * and the submit field, named elecname is set to the name of the elective.
         * the field FormId is set to "ElectiveMain" (ie, in Student's main 
         * page, forms for electives
         *
         * The Form Send: The hidden field ElectiveNo is used to identify the Elective
         * and the field FormId is set to "ElectivePage". For each course there is one hidden field
         * with name "Courses" and value cno, and a set of (6) radio buttons,  with name 
         * cno and values  0(for no pref),1,..,5
         *
         */

        // show all courses of the elective 
        String elecno = (String) req.getParameter("elecno");
        String elecname = (String) req.getParameter("elecname");
        String query = "select course.cno, course.cname, course.curl, faculty.furl,faculty.name"
            + " from elective,course,faculty"
            + " where elective.cno = course.cno"
            + " and elective.eno = '" + elecno + "'"
            + " and faculty.fno = course.fno ";
        //System.err.println(query);
        
        Statement  stmt = conn.createStatement();
        ResultSet rset = stmt.executeQuery(query);
        // start the form
        out.println("<h1>Courses for the Elective "+elecname+"<br></h1>"
              + "<hr>Please indicate your preference against at most "
              + "5 courses<br>( pref-no 1 means highest preferece )<br><hr>");
        out.println("<form name=Elective"
              + " action=\"http://everest.cse.iitb.ernet.in:5024/servlet/Main\" "
              + " method=post> <br> " 
              + " <input type=hidden name=FormId value=ElectivePage>"
              + " <input type=hidden name=ElectiveNo value="+elecno+">");

        while (rset.next()) {
            // print course name with five + one options
            String cno = rset.getString("cno");
            out.println( "<input type=hidden name= Courses value="+cno+">"
                +" None<input type=radio name= "+cno+" value=0>"
                +" 1<input type=radio name= "+cno+" value=1>"
                +" 2<input type=radio name= "+cno+" value=2>"
                +" 3<input type=radio name= "+cno+" value=3>"
                +" 4<input type=radio name= "+cno+" value=4>"
                +" 5<input type=radio name= "+cno+" value=5>"
                + "<a href=" + rset.getString("curl") + ">"
                + cno + " " + rset.getString("cname") + "</a>"
                + " <a href=" + rset.getString("furl") + ">"
                + rset.getString("name") + "</a><br>");

        }

        out.println("<input type=submit value=Submit> " +
                "<input type=reset type=reset value=\"Reset Form\"><br></form>");

        stmt.close();
        

    }

    /** This is the function that is called from outside when the session 
      * starts.  Not called through processQuery 
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
     */
    public void printFirst(PrintWriter out) throws SQLException
    {
        int phase = Configuration.getPhase();
        switch(phase) {
            case 0: printMost(out);
                    printEnd(out);
                    printElectives(out);
                    break;
            case 1:
                    printMost(out);
                    printEnd(out);


                    printElectives(out);
                    printWarnings(out);
                    break;

            case 2:
                  //  out.println("Phase 2: What to printFirst?");
                 // foll prints what printMost prints w/o checkboxes
                    printMost2(out);
                    break;
        }
        return;
    }


    /** This method is called from outside to process a request.
      * Depending on the current phase calls other functions 
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    public void processQuery(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {
        int phase = Configuration.getPhase();
        switch(phase) {
            case 0:processQueryPhase0(req,out); break;
            case 1:processQueryPhase1(req,out); break;
            case 2:processQueryPhase2(req,out); break;
            default: /* What to do? */ ;
        }
    }


    /** Method to process phase 0 requests. The three forms that may
      * come are StudentMain, ElectiveMain ( from Student Main Page )
      * and ElectivePage 
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        executing a query on database
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void processQueryPhase0(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {


            
        String FormId=(String) req.getParameter("FormId");
            
        if(FormId!=null)  {
            
            if ( "StudentMain".equals(FormId) ) {
                mainpagePhase0(req,out);
            }
            else if ( "ElectivePage".equals(FormId) ){
                // to process the Elective Page
                electivePhase0(req,out);
            }
            else if ( "ElectiveMain".equals(FormId) ) {
                // to process the Elective form in the main page
                sendElective(req,out);
            }

            else {
                out.println("<H1>Illegal Form</H1><br>"
                            + " The form you have submitted is invalid!"
                            + " Please try again! <br>");
            }
        } else  {
            out.println("<H1>Illegal Form</H1><br>"
                        + " The form you have submitted is invalid!"
                        + " Please try again <br>");
        }

        return;
    }


    /** To search for a course by no/name and print it with checkboxes .
      * calls printSearch to do the printing 
        @param out Printwriter to write html stuff to.
        @param search_cno which course number to search .
        @param search_cname which course name to search.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void do_and_printSearch(PrintWriter out,String search_cno,String search_cname)
        throws SQLException
    {

        boolean cno, cname;     // what to search: cno? cname? both? neither?

        cno = search_cno != null && (search_cno.compareTo("")!=0); 
        cname = search_cname != null && (search_cname.compareTo("")!=0);

        if (!cno && !cname ) return;
        
        search_cno=search_cno.replace('*','%');
        search_cno=search_cno.replace('?','_');

        search_cname=search_cname.replace('*','%');
        search_cname=search_cname.replace('?','_');


        String search_qry="select unique course.cno, course.cname, course.curl,"
            + " faculty.furl, faculty.name "
            + " from course, faculty where (" 
            + (cno?(" course.cno like '"+search_cno+"'"):(""))
            + ((cno&&cname)?(" or "):(""))
            + (cname?(" course.cname like '" + search_cname + "'"):(""))
            + ") and faculty.fno = course.fno ";
            //System.err.println("Searching Query: "+search_qry);
            Statement search_stmt=conn.createStatement();            
            ResultSet rset=search_stmt.executeQuery(search_qry);
            
            printSearch(out,rset);
            search_stmt.close();

    }


    /** Method to process the form StudentMain in phase 0. Updates
      * the registration table for the student using the form submitted
      * and sends back the modified course list. 
      * Also processes the search requests ( by calling do_and_printSearch )
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void mainpagePhase0(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {

        String[] selectedCourses = req.getParameterValues("registration");
        
        String query = "delete from registration "
            +"where srollno = '"
            + idno +"'";
        Statement  stmt = conn.createStatement();
        stmt.executeUpdate(query);
        stmt.close();
        
        int numselcrs = (selectedCourses==null)?0:selectedCourses.length;
        for(int i = 0; i < numselcrs; i++)
        {
            Statement  insstmt = conn.createStatement();
            String insquery = "insert into registration values('"
                + idno + "','" + selectedCourses[i] + "')";
            //System.err.println("Insertion query: "+insquery);
            insstmt.executeUpdate(insquery);
            insstmt.close();
        }

        // now print the html page
        printMost(out);
        
        String search_cno=(String) req.getParameter("cno_search");
        String search_cname = (String) req.getParameter("cname_search");
        
        do_and_printSearch(out,search_cno,search_cname);           

        printEnd(out);
        // Now the electives
        printElectives(out);
    }

    /** To process the elective page in phase 0. The form submitted has the preferences
     * for a particular elective. Get the elective name and
     * courses/preferences and insert into the table "elective_prefs" 
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
     */
    private void electivePhase0(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {
        
        /*
         * The Form Received: The hidden field ElectiveNo is used to identify the Elective
         * and the field FormId is set to "ElectivePage". For each course a set of (6) radio 
         * buttons with name "elecno" and value set to cno are there. There values are 0,1,..,5
         * 
         * Sends back the StudentMainPage ( by calling printFirst() )
         *
         */
            String electno =req.getParameter("ElectiveNo");
            String delqry = "delete from elective_prefs where"
                + " srollno ='" + idno + "' and eno = '"
                + electno + "'";
            
            Statement  delstmt = conn.createStatement();
            delstmt.executeUpdate(delqry);
            delstmt.close();

            String[] cnos = req.getParameterValues("Courses");

            int len = ( cnos == null )? 0: cnos.length;
            for ( int i=0; i < len; i++ ) {
                String prefno = req.getParameter(cnos[i]);
                if ((prefno!=null) && (!prefno.equals("0"))) {
                    String query = "insert into elective_prefs values ( '"
                        + idno + "', '" + electno + "','" + cnos[i] + "', "  
                        + prefno + ")";
                    Statement  stmt = conn.createStatement();
                    stmt.executeUpdate(query);
                    stmt.close();
                }
            }
            printFirst(out);

}

    /** Method to process phase 1 queries. Just like processQueryPhase0,
      * but prints warnings also for courses with timetable clashes 
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void processQueryPhase1(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {
            processQueryPhase0(req,out);
            printWarnings(out);
    }

    /** Method to send a list of courses taken by the student which are 
      * in the same slot.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void printWarnings(PrintWriter out) 
        throws SQLException
    {
            String query = " select unique c.cno,c.cname,c.cslot from"
            + " ( select course.cno, course.cname, course.cslot from"
                    + " course,registration where"
                    + " course.cno = registration.cno"
                    + " and registration.srollno = '" + idno + "'"
                    + " ) c ,"

            + " ( select course.cno, course.cslot from"
                    + " course,registration where"
                    + " course.cno = registration.cno"
                    + " and registration.srollno = '" + idno + "'"
                    + " ) cc"
            + " where c.cslot = cc.cslot"
            + " and c.cno <> cc.cno ";
            
                
            // System.err.println(query);

            Statement  stmt = conn.createStatement();
            ResultSet rset = stmt.executeQuery(query);
            if (rset.next()) {
                out.println("<br><br><i><h2>Warning! The following courses "
                    + "have a slot clash!</h2> </i><br>");
                do {
                   out.println("<h3>"+rset.getString("cno")+" "+ 
                       rset.getString("cname")+ " " +
                       "Slot: " + rset.getString("cslot")+"</h3>  <br>");
                   } while (rset.next());
            }
            
    }

    /** To process a phase 2 query. If a search request comes, shows the search results
      * ( using do_and_printSearch ) and also shows the other info ( using printMost2 ).
      * If a request for consent comes adds it to the regconsent table after proper validation.
        @param req request from user when she submits a form.
        @param out Printwriter to write html stuff to.
        @exception SQLException thrown if we go wring somewhere while
        executing a query on database
      */
    private void processQueryPhase2(HttpServletRequest req,PrintWriter out)
        throws SQLException
    {

        /* Forms received: 
                1. FormId = "Search".
                   two fields cno_search and cname_search;
                2. FormId = "ReqConsent" 
                   a list of parameters, name = "registration", value = cno
           Form send back:
                Do printMost2 ( with search box ).
                Search Results ( as "ReqConsent" form ) with checkboxes
                unselected 
         */

        String FormId = req.getParameter("FormId");
        if ( FormId == null ) {
            // error mesg
            out.println("<h1>Illegal Form</h1><br>The form you submitted "
                        + " is not valid! Please try again<br> ");
            return;
        }


        if (FormId.equals("Search")) {
            
            
            // search form: send back check boxes for all the courses searched for

            String search_cno=(String) req.getParameter("cno_search");
            String search_cname = (String) req.getParameter("cname_search");

            // open the form "ReqConsent"

            out.println ("<form name=ConsentRequest"
               + " action=\"http://everest.cse.iitb.ernet.in:5024/servlet/Main\" "
               + " method=post> <br> " 
               + " <input type=hidden name=FormId value=ReqConsent>"
               + " <h1>Request for Consent to register</h1><br>");

            // this function searches and prints the search results with checkboxes
            // the name of the checkboxes is "registration" ( later change if needed )
            do_and_printSearch(out,search_cno,search_cname);
        
            out.println("<input type=submit value=\"Request Consent\"> " +
                "<input type=reset type=reset value=\"Reset Form\"><br></form>");
            out.println ("</form><br><hr>");
            

            // just show the courses 
            printMost2(out);
         
         }

         else if (FormId.equals("ReqConsent")) {

                
            Vector regdCourses = new Vector(); 
            Vector againCourses = new Vector(); 
             
            String[] reqCourses = req.getParameterValues("registration");
            int numreqcrs = (reqCourses==null)?0:reqCourses.length;
            for(int i = 0; i < numreqcrs; i++)
            {
                // first check if the student is registered for that course
                String chkqry = "select * from registration where srollno = '"
                                + idno + "' and cno = '" + reqCourses[i] + "'";
                
                Statement chkstmt = conn.createStatement();
                ResultSet chkrs =  chkstmt.executeQuery(chkqry);

                if (chkrs.next()) {
                    regdCourses.addElement(reqCourses[i]);
                }

                else {
                    Statement  insstmt = conn.createStatement();
                    String insquery = "insert into regconsent values('"
                        + idno + "','" + reqCourses[i] + "',0,0)";
                    //System.err.println("Insertion query: "+insquery);
                    try {
                        insstmt.executeUpdate(insquery);
                    } catch (SQLException s) {
                        againCourses.addElement(reqCourses[i]);
                        
                        
                    }
                    insstmt.close();
                }
            }

            if (regdCourses.size() > 0) {
                    // that means some of the courses requested are already registered for
                    out.println("<hr><h3>Illegal Requests</h3><br> You have already " 
                            + " registered for the following course(s) for "
                            + " which consent to register is requested.<br>");
                    int sz = regdCourses.size();
                    for ( int i = 0; i< sz; i++ ) {
                        out.println(regdCourses.elementAt(i) + "<br>");
                    }
            }

            
            if (againCourses.size() > 0) {
                    // that means some of the courses requested are already requested for
                    out.println("<h3>Illegal Request</h3> <br> " 
                                 + " It seems you have submitted invalid information."
                                 + " This could mean requesting consent for a course "
                                 + " for which you have already submitted the request.<br> ");
                    int sz = againCourses.size();
                    for ( int i = 0; i< sz; i++ ) {
                        out.println(againCourses.elementAt(i) + "<br>");
                    }
            }


            // and now show the courses 
            out.println("<hr>");
            printMost2(out);
         
             
         }

                
    }

    
}

